% <md.m> Mark D Shattuck 1/22/1996
% Molecular Dynamics Simulator
% 01/22/96 Mark D. Shattuck <mds> md.m  
%          md power law potentials (L-J)
%          000 euler
% 01/31/96 001 mds velocity verlet
% 02/22/96 006 mds hertz law
% 05/15/00 010 mds add energy loss and gravity
% 07/15/04 020 mds setup for pvnrt sim
% 09/08/04     mds changed force law to eq. 9 in J. Phys. I 6 (1996) p. 5-20
% 09/09/04 021 mds added eq 13
% 10/05/04 022 mds/rm add rotations
% 10/12/04 023 mds/rm add static wall friction and correct ball-bal friction
% 12/12/04 024 mds add restart and full particle saving
% 10/20/08 Mark D. Shattuck <mds> mdsp.m
%          000 setup for spatial dependent forcing.
%              Vector diameters.
% 01/06/09 Mark D. Shattuck <mds> mdhopper.m
%          000 setup for hopper.
%          001 rationalize geometry
% 01/16/09 Mark D. Shattuck <mds> mdyale.m
%          001 setup for Yale geometry.
%          002 clean up
%          005 friction tap
%          007 compress

%% Error Handling
% clear all;
% err=41;
% save('pers','err');
% 
% while(1); % try again no matter what
% 
% try
  
%% Clear All

% clear all;
%load('pers');

%% Constants

g=9.81;  %gravity in m/s

%% Initialize particle and wall properties

sig0=(.4994)*2.54/100; % particle diameter scale m
%G=4/3;
G=1.252;
m=0.265e-3;          % particle mass in kg
Io=1/2;             % particle moment of interia divided by MR^2 Io=I/M/R/R
Eyp=4e9;           % Young's Modulus of particles in Pa 200GPa for steel 0.05GPa for rubber
nup=.5;             % Poisson's ratio of particles, for rubber it's 0.5 
sigw=10*sig0;        % wall effective diameter;
Wf=20;
M=Wf*m;            % Top Plate mass in kg
Eyw=Eyp;            % Young's Modulus of w
nuw=nup;            % Poisson's ratio of w
Eeffpp=1/(2*(1-nup^2)/Eyp);         % Effective Ey for particles
Reffpp=(sig0/2*sig0/2)/(sig0/2+sig0/2); %Effective Radius
MR=M/m;             % ratio of masses

K0=4/3*sqrt(Reffpp)*Eeffpp/m;      % from eq. 9 in J. Phys. I 6 (1996) p. 5-20 K=k_n~/m (N/m^(3/2)/kg)
Q0=190/m;       % strength of normal friction from eq. 9 in J. Phys. I 6 (1996) p. 5-20 Q=gamma_n~/m (1/m^(1/2)/s)

K=K0;        % Actuall K
Kw=K0;  % strength of spring for walls (m/s/s ?)     

%Q0=0;    %check the energy conservation 
Q=Q0;       % Actuall 
Qw=Q0;       % strength of normal friction for top and side walls
Qwb=Q0;       % strength of normal friction for bottom plate

mu=0.8;      % ball-ball kinetic friction
muw=0.4;     % ball-wall kinetic friction
beta=0;      % top plate friction
mu=0;
muw=0;
alpha=1.5;   % force exponent alpha=1 (linear) alpha=3/2 (Hertz)

%% Driving Parameters
gam=0;      % non-dimensional acceleration of bottom wall
f=30;        % frequency of bottom wall in Hz
w=2*pi*f;
w1=0;
A0=(gam+0)*g/w/w; % max A
Nstep=60;     % # of steps up same # down

%% Initialize Simulation Parameters

dt=3e-5;      % Time step in seconds

Tr1=.1*1.05;       % Time for randomize
Td1=.2*1.05;       % Time for 1st decay with friction
Td2=12*1.05;        % Time for 2nd decay without friction

Texp=Tr1+Td1+Td2;  % total experiment time

Nexp=1;     % Number of experiments
Nps=1;        % Number of experiments per set
Nset=Nexp/Nps; % Number of sets

Npic=400;   % Number of pictures during decay

npskip=35*10;       % Number of steps to skip before ploting/printing

% convert times to time steps
Ntr1=fix(Tr1/dt)+1;
Ntd1=fix(Td1/dt)+1;
Ntd2=fix(Td2/dt)+1;
Ntexp=Ntr1+Ntd1+Ntd2;
Nppic=fix(Ntd2/Npic);

ntmax=Ntexp*Nexp;       % Total number of time steps

basename=sprintf('Yale_compress%03d[%03d]',200,statenum);              % Base name for saved files
%basename='test';              % Base name for saved files
plotit=true;                     % Show plot ?
PhotoElastic=false;              % plot Photo elastic image
%PhotoElastic=true;               % plot Photo elastic image
fimg=[basename '.img'];          % PhotoElastic images
Ny=200;                          % PE horizontal size
Nx=180;                          % PE vertical size

%% Experimental conditions

Ns=4;
Nl=3;
NP=Ns+Nl;              % Number of particles
Np=NP;                 % current number
sig=sig0*([ones(1,Ns) G*ones(1,Nl)]);
poly=0;
sig=sig.*[1-poly/2+poly*(0:Ns-1)/(Ns-1) 1-poly/2+poly*(0:Nl-1)/(Nl-1)];
MM=(sig/sig0).^2;

Dnm=repmat(sig,Np,1);
Dnm=(Dnm+Dnm')/2;
Dnm(eye(Np)==1)=0;

T=0;                 % Initial temperature
H=sig0/.23512;       % Height of cell
TH=1.1*sig0*G;       % z-Thickness of cell
B=8*sig0;            % size of bottom
% B=sig0*2.6512;      % size of bottom
P=0;                 % size of hole
Theta=0*pi/180;      % hopper angle
W=H*tan(Theta);      % x-size of wedge
F=0;                 % oulet hieght
Fc=A0+G*sig0/10;     % oulet hieght
Ly=H+F;          % y-size Computational domain
Lx=2*W+B;        % x-size Computational domain
Lz=TH;                % z-size Computational domain
Xc=Lx/2;              % center of domain
Yc=Ly/2;
Zc=Lz/2;

Lxp=5*sig0;       % Size to plot
Lyp=Ly;
Xop=0;       % origin of plot
Yop=0;

%% Cell List Setup

NBx=2;        % # of particles in a cell
NBy=1;        % # of particles in a cell
NBz=1;              
scx=Lx/NBx;            % Scale for cells
scy=Ly/NBy; 
scz=Lz/NBz;


%% Initial Conditions

% particle postions

% Hexagonal Close Packed
R=2;
C=4;
[x y z]=crystal(R,C,1);  % 0,0,0 at center of cyrstal
yt=y+((C-1)/2*sqrt(3)+1+5)/2;
ii=find(yt(:)>1);
x=x(ii);
y=y(ii);
z=z(ii);
[jj ii]=sort(rand(1,numel(x)));
ii=ii(1:Np);
x=x(ii);
y=y(ii);
z=z(ii);
% move to edge of crystal   
y=y+((C-1)/2*sqrt(3)+1)/2;
x=x(:).'*sig0*G*1+Xc;
y=y(:).'*sig0*G*1+F;
z=z(:).'*sig0*G*1+Zc;

x=(ppy(:,statenum)'+.5)*Ly;
y=(ppx(:,statenum)'+.5)*Ly;


% % Random locations on a grid;
% Nx=ceil(sqrt(Np));                  % Number of grid points
% Ny=ceil(Np/Nx);
% [xx yy]=ndgrid(.5:Nx-.5,.5:Ny-.5);  % Create grid
% [junk ii]=sort(rand(1,Nx*Ny));      % Place particle at random
% x=Lx/2*xx(ii(1:Np))/Nx*2;           % Initial positions
% y=Ly/2*yy(ii(1:Np))/Ny*2+2*A0;

% particle velocities
vbar=T^(1/2);                       % RMS velocity m/s
vx=vbar*randn(1,Np);              % Initial Velocities
vy=vbar*randn(1,Np);
vz=zeros(1,Np);

% particle acceration
ax=0*vx;
ay=0*vy;
az=0*vz;

% particle force matrix
Fx=zeros(Np,Np+6);
Fy=zeros(Np,Np+6);
Fz=zeros(Np,Np+6);

ax_old=0*vx;   % Initial Accelerations
ay_old=0*vy-g;
az_old=0*vz;

% rotations
xr1=0*rand(1,Np)*pi*sig0;
xr2=0*rand(1,Np)*pi*sig0;
xr3=0*rand(1,Np)*pi*sig0;
vr1=0*randn(1,Np);
vr2=0*randn(1,Np);
vr3=0*randn(1,Np);
ar1=zeros(1,Np);
ar2=zeros(1,Np);
ar3=zeros(1,Np);
ar1_old=0*ar1;
ar2_old=0*ar2;
ar3_old=0*ar3;

% Save at each time step
ytps=zeros(1,Ntexp);
Ftps=zeros(1,Ntexp);
Ek=zeros(1,Ntexp);
mxv=zeros(1,Ntexp);
Ep=zeros(1,Ntexp);
Eg=zeros(1,Ntexp);

% Top plate
ytp=Ly;
ytps(1)=ytp;
vytp=0;  
aytp=-g;
aytp_old=-g;

% Bottom plate
ybp=A0;
vybp=A0*w;
aybp=0;
aybp_old=0;
ayw=0;


%% Save inital state
save([basename '_info']);

%% Plot Initial Conditions
if (plotit==true);
  figure(1);
  clf;
  if(~PhotoElastic)
    Nth=32;
    thp=(0:Nth)/Nth*2*pi;
    sth=sin(thp)/2;
    cth=cos(thp)/2;
    hp=zeros(1,Np);
    h=plot([W Xc-P/2]/sig0,(F+ybp)*[1 1]/sig0,'r-',[Xc+P/2 W+B]/sig0,(F+ybp)*[1 1]/sig0,'r-',[0 W]/sig0,[F+H F]/sig0,'r-',[2*W+B W+B]/sig0,[ytp F]/sig0,'r-',[W W+B]/sig0,(F+ytp)*[1 1]/sig0,'k-');
    hold on;
    %hl=plot(([(x-sig.*cos(xr3./sig*2)/2);(x+sig.*cos(xr3./sig*2)/2)]/sig0),([(y-sig.*sin(xr3./sig*2)/2);(y+sig.*sin(xr3./sig*2)/2)]/sig0),'b-');
    for np=1:Np
      hp(np)=plot((x(np)+sig(np)*sth)/sig0,(y(np)+sig(np)*cth)/sig0,'b','linewidth',1);
    end
    axis([Xop+[0 Lxp] Yop+[0 Lyp]]/sig0);
    set(gca,'DataAspectRatio',[1 1 1]);
    set(h,'erasemode','none');
    set(h,'erasemode','xor');
    drawnow;
    view(90,-90);
  else
    xr3=xr3*0+2*pi*rand(size(xr3));
    sigb=[sig 0];
    [yy xx]=ndgrid((1:Nx),(1:Ny));
    s1=angle(-i*((xx+i*yy)/Ny*Lxp-W+Xop+i*(-F+Yop)))<Theta;
    s2=angle(-i*((xx+i*yy)/Ny*Lxp-W-B+Xop+i*(-F+Yop)))>-Theta;
    o1=(yy/Ny*Lxp-F+Yop)>0;
    o2=(abs((xx/Ny*Lxp-Xc+Xop))<P/2);
    bg=flipud(conv2((s1 & s2).*o1+o2.*(1-o1),ones(3,3),'same')/9/5);
    sc=1+max(bg(:));
    fid=fopen(fimg,'wb');
    clf;
    if(Np>0)
      [im cr over]=ipe(Fx,Fy,x-Xop,y-Yop,sig,Lx,Theta,ybp,ytp,Lxp,Ny,Nx);
      qq=sin(.01/2*im).^2;
      img=max(qq,ipf(abs(cr),1,.025)-.98*ipf(abs(cr),.98,.025))+bg;
      xr3b=[xr3./sig*2 0];
      img=(img+.3*ipf(abs(cr),1,.025).*ipf(abs(real(cr.*sigb(over).*exp(-i*xr3b(over)))),.15*sig0,.03*sig0));
    else
      img=bg;
    end
    simage(img);
    caxis([0 sc]);
    drawnow;
    fwrite(fid,uint8(clip(img*255/sc,0,255)),'uint8');
  end
end

%% Calculate Initial Potential and Kinetic Energy

dx=repmat(x,Np,1);
dx=dx-dx';
dy=repmat(y,Np,1);
dy=dy-dy';
dz=repmat(z,Np,1);
dz=dz-dz';
delta=Dnm-sqrt(dx.^2+dy.^2+dz.^2);   %dx,dy,dz are matrices
Ek(1)=sum(vx.^2+vy.^2+vz.^2)/2;   % Kinetic Energy
EWk(1)=MR*vytp.^2/2;              % Top Plate Kinetic Energy
Eg(1)=sum(g*y);                   % Gravitational Potential Energy
EWg(1)=MR*g*ytp;                  % Top Plate Gravitational Potential Energy
Ee(1)=sum(K*2/5*(delta(:)>0).*delta(:).^(5/2))/2; % Elastic energy (needs to be changed for alpha)
EWe(1)=0;                                         % Top Plate Elastic energy 
Ekr(1)=Io*sum(vr1.^2+vr2.^2+vr3.^2)/2;            % Rotational Kinetic energy
E0=Ek(1)+EWk(1)+Ee(1)+EWe(1)+Eg(1)+EWg(1)+Ekr(1); % Total Energy
mxv(1)=0;

%% Initialize save variables

px=zeros(Np,Nps,Npic+1);
py=zeros(Np,Nps,Npic+1);
Fxo=zeros(Np,Np+6,Nps,Npic+1);
Fyo=zeros(Np,Np+6,Nps,Npic+1);


%% Initialize Loop

nt=1;
ns=0;
nw=0;
t0=clock;
elapt=etime(clock,t0);
fprintf(1,'%s %s %s %s %s %s %s %s\n','ytp(nt)','Ek(nt)+EWk(nt)','Ekr(nt)','Eg(nt)+EWg(nt)','Ee(nt)+EWe(nt)','Ek(nt)+Ee(nt)+Eg(nt)+EWk(nt)+EWe(nt)+EWg(nt)+Ekr(nt)', 'elapt/60', 'elapt/nt*(ntmax-nt)/60');
lnt=nt;
t=0;
tEWe=0;
tEe=0;
A=0;
mu0=mu;
muw0=muw;
state=2;
nset=1;

%% Main Loop
dt0=dt;
slomo=1;
dytp=0;
ntf=nt;
slof=1;
while(nt<=ntmax)
  ytp0=ytp;
  
  te=rem(t,Texp);
  expN=fix((nt-1)/Ntexp)+1;
  nexp=mod(expN-1,Nps)+1;
  ntexp=mod(nt-1,Ntexp)+1;
  if(ntexp<=Ntr1)
    if(state~=0); 
      state=0; 
      npic=1;
      dt=dt0;
      K=K0/1000;
      Kw=K0/1000;
      Q=Q0;
      Qw=Q0;
      Qwb=Q0;
    end
    if(ntexp<Ntr1/2)
      if(sin(2*pi*te/.01)>2)
        ax_old=ax_old+vbar*randn(1,Np)/dt;
        ay_old=ay_old+vbar*randn(1,Np)/dt;
      end
    else
      mu=0;
      muw=0;
      K=K0/100;
      Kw=K0/100;
    end      
  elseif(ntexp>Ntr1 && ntexp<=Ntr1+Ntd1)
    if(state~=1); 
      state=1; 
      dt=dt0/3;
      K=K0;
      Kw=K0;
      Q=Q0*30;
      Qw=Q0*30;
      Qwb=Q0*30;
      rdt=Ntr1+Ntd1;
    end
    %ax_old=ax_old+vbar*randn(1,Np);
  elseif(ntexp>Ntr1+Ntd1)
    if(state~=2); 
      state=2; 
      mu=0;
      muw=0;
      dt=dt0/3;
      K=K0;
      Kw=K0;
      Q=Q0*30;
      Qw=Q0*30;
      Qwb=Q0*30;
      vytp=-Ly/Ntd2/dt;
    end
    if(ntexp-Ntr1-Ntd1==ceil(Ntd2/2)+1)
      vytp=Ly/Ntd2/dt;
    end
    if(ntexp-Ntr1-Ntd1<Ntd2/2)
      ytp0=Ly-2*Ly/2*(ntexp-Ntr1-Ntd1)/Ntd2;
    else
      ytp0=Ly/2+2*Ly/2*(ntexp-Ntr1-Ntd1-Ntd2/2)/Ntd2;
    end  
    %ax_old=ax_old+vbar*randn(1,Np);
    %dt=min(dt0/3,1000/max(abs(ay)));
    if(mod(ntexp-Ntr1-Ntd1-1,Nppic)==0)
      npic=fix((ntexp-Ntr1-Ntd1-1)/Nppic)+1;
      fprintf(1,'Storing [%04d][%04d]...',nexp,npic);
      px(:,nexp,npic)=x;
      py(:,nexp,npic)=y;
      Fxo(:,:,nexp,npic)=Fx;
      Fyo(:,:,nexp,npic)=Fy;
      fprintf(1,'done.\n');
    end
  end
  if(mod(nt,Nps*Ntexp)==0)
        sfn=sprintf('%s[%04d]',basename,nset);
        ssfn=sprintf('%s[%04d]s',basename,nset);
        fprintf(1,'Saving %s...',sfn);
        save(sfn,'px','py','Fxo','Fyo','ytps','Ftps','Ek','Ep','Eg','mxv');
        sytps=mean(reshape(ytps(Ntd1+Ntr1+(1:Ntd2-1)),1050,400));
        sFtps=mean(reshape(Ftps(Ntd1+Ntr1+(1:Ntd2-1)),1050,400));
        save(ssfn,'px','py','sytps','sFtps');
        fprintf(1,'done.\n');
        nset=nset+1;
%        pause;
  end

  A=A0;   % Shaking Amplitude

  % ploting, saving and other user feed back  
  if (mod(nt-1,npskip*slomo+slomo-1)==0)
    if (plotit==true);
      figure(1)
      if(~PhotoElastic)
        for np=1:Np  %---draw the inner line to each circle
          set(hp(np),'xdata',(x(np)+sig(np)*sth)/sig0,'ydata',(y(np)+sig(np)*cth)/sig0);
         % set(hl(np),'xdata',[(x(np)-sig(np)*cos(xr3(np)./sig(np)*2)/2)/sig0; (x(np)+sig(np)*cos(xr3(np)./sig(np)*2)/2)/sig0],'ydata',[(y(np)-sig(np)*sin(xr3(np)./sig(np)*2)/2)/sig0; (y(np)+sig(np)*sin(xr3(np)./sig(np)*2)/2)/sig0]);
        end % we want it to be greater than or equal to 4
        set(h(1),'ydata',(F+ybp)*[1 1]/sig0);
        set(h(2),'ydata',(F+ybp)*[1 1]/sig0);
        set(h(end),'ydata',(F+ytp)*[1 1]/sig0);
        drawnow;
        view(90,-90);
      else
        sk=1;
        if(mod(fix(nt/npskip),sk)==0)
          clf;
          if(Np>0)
            [im cr over]=ipe(Fx,Fy,x-Xop,y-Yop,sig,Lx,Theta,ybp,ytp,Lxp,Ny,Nx);
            qq=sin(.01/2*im).^2;
            %        simage(qq+(ipf(abs(cr),1,.025)-.95*ipf(abs(cr),.98,.025)+randn(256,110)/50)+(1-ipf(yy,2*(256-ytp/Lx*110),1))/10);
            img=max(qq,ipf(abs(cr),1,.025)-.98*ipf(abs(cr),.98,.025))+bg;
            xr3b=[xr3./sig*2 0];
            img=(img+.3*ipf(abs(cr),1,.025).*ipf(abs(real(cr.*sigb(over).*exp(-i*xr3b(over)))),.15*sig0,.03*sig0));
          else
            img=bg;
          end

          simage(img);
          caxis([0 sc]);
          fwrite(fid,uint8(clip(img*255/sc,0,255)),'uint8');
          drawnow;
        end
      end
    end
      if (mod(nt-1,npskip)==0)

    fprintf('[%04d][%04d][%d]|%05d|',statenum,inns(statenum),state,fix(nt/npskip));
    elapt=etime(clock,t0);
    fprintf(1,'%f %f %f %f %f\n',ytp,sum(vr1),sum(vx),sum(vy),elapt/nt*npskip);
    %fprintf(1,'%f \n',max(Fy(:)));
      end
  end

  x=x+vx*dt+ax_old.*dt.^2/2;  % first step in Verlet integration
  y=y+vy*dt+ay_old.*dt.^2/2;
  z=z+vz*dt+az_old.*dt.^2/2;
  
  xr1=xr1+vr1*dt+ar1_old.*dt.^2/2;
  xr2=xr2+vr2*dt+ar2_old.*dt.^2/2;
  xr3=xr3+vr3*dt+ar3_old.*dt.^2/2;

  %top plate positions
  ytp = ytp + vytp*dt + aytp_old*dt^2/2;
  
  Ek(ntexp)=sum(vx.*vx +vy.*vy+vz.*vz)/2;
  Eg(ntexp)=sum(g*x);

  ar1=zeros(1,Np);
  ar2=zeros(1,Np);
  ar3=zeros(1,Np);
  ax=zeros(1,Np);
  ay=zeros(1,Np);
  az=zeros(1,Np);
  aytp=0;

  Fx=0*Fx;
  Fy=0*Fy;
  Fz=0*Fz;
  
  tEWe=0;
  
  % angled x-walls
  deltaw=sig/2-(x-W)*cos(Theta)-(y-F)*sin(Theta);
  yt=F+ybp-y;
  jj=find((deltaw>=0)  & (yt<0)); %returns the indices where this is true
  if(~isempty(jj))
    deltaw=2*deltaw(jj);
    dvnw=vx(jj)*cos(Theta)+vy(jj)*sin(Theta);                %relative velocity of the wall to the particle in normal direction(the normal direction is the positive direction)  dvnw=(Vw-Vp)*Nw=-Vp*(-nx)=Vx
    das2w=-vz(jj)-vr2(jj)*cos(Theta)+vr1(jj)*sin(Theta);      %unit vector of angular acceleration direction in y component
    das3w=vy(jj)*cos(Theta)-vx(jj)*sin(Theta)-vr3(jj);       %unit vector of angular acceleration direction in z component
    aew=-(Kw.*deltaw.^alpha);   %acceleration of the particle due to elasticity of wall
    afw=Qw*dvnw.*sqrt(deltaw);  %acceleration of the particle due to friction of wall    the directions of aew & afw are related with the normal direction
    aw=aew+afw;                 %total normal acceleration of the particle(the normal direction is the positive direction)
    das23w=sqrt(das2w.^2+das3w.^2);
    mm=das23w==0;        
    das23w(mm)=1;               %to avoid the denominator to be zero
    as2=muw*(das2w./das23w).*abs(aw);
    as3=muw*(das3w./das23w).*abs(aw);
    ax(jj)=ax(jj)+(-aw*cos(Theta)+as3*sin(Theta));           %ax=ax+aw*Nw=ax-aw
    ay(jj)=ay(jj)+(-aw*sin(Theta)-as3*cos(Theta));
    az(jj)=az(jj)+as2;
    Fx(jj,Np+1)=(-aw*cos(Theta)+as3*sin(Theta));
    Fy(jj,Np+1)=(-aw*sin(Theta)-as3*cos(Theta));
    Fz(jj,Np+1)=as2;
    ar1(jj)=ar1(jj)+as2/Io*sin(Theta);
    ar2(jj)=ar2(jj)+as2/Io*cos(Theta);
    ar3(jj)=ar3(jj)+as3/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
  deltaw=sig/2+(x-W-B)*cos(Theta)-(y-F)*sin(Theta);
  ii=find((deltaw>=0)  & (yt<0)); %returns the indices where this is true
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=-vx(ii)*cos(Theta)+vy(ii)*sin(Theta);                %relative velocity of the wall to the particle in normal direction(the normal direction is the positive direction)  dvnw=(Vw-Vp)*Nw=-Vp*(-nx)=Vx
    das2w=-vz(ii)+vr2(ii)*cos(Theta)+vr1(ii)*sin(Theta);      %unit vector of angular acceleration direction in y component
    das3w=-vy(ii)*cos(Theta)-vx(ii)*sin(Theta)-vr3(ii);       %unit vector of angular acceleration direction in z component
    aew=-(Kw.*deltaw.^alpha);   %acceleration of the particle due to elasticity of wall
    afw=Qw*dvnw.*sqrt(deltaw);  %acceleration of the particle due to friction of wall    the directions of aew & afw are related with the normal direction
    aw=aew+afw;                 %total normal acceleration of the particle(the normal direction is the positive direction)
    das23w=sqrt(das2w.^2+das3w.^2);
    mm=das23w==0;        
    das23w(mm)=1;               %to avoid the denominator to be zero
    as2=muw*(das2w./das23w).*abs(aw);
    as3=muw*(das3w./das23w).*abs(aw);
    ax(ii)=ax(ii)+(aw*cos(Theta)+as3*sin(Theta));           %ax=ax+aw*Nw=ax-aw
    ay(ii)=ay(ii)+(-aw*sin(Theta)+as3*cos(Theta));
    az(ii)=az(ii)+as2;
    Fx(ii,Np+2)=(aw*cos(Theta)+as3*sin(Theta));
    Fy(ii,Np+2)=(-aw*sin(Theta)+as3*cos(Theta));
    Fz(ii,Np+2)=as2;
    ar1(ii)=ar1(ii)+as2/Io*sin(Theta);
    ar2(ii)=ar2(ii)-as2/Io*cos(Theta);
    ar3(ii)=ar3(ii)+as3/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
  
  % outlet x-walls
  deltaw=sig/2-(x-W+(B-P)/2);
  ii=find((deltaw>=0)  & (yt>=0)); %returns the indices where this is true
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=vx(ii);                %relative velocity of the wall to the particle in normal direction(the normal direction is the positive direction)  dvnw=(Vw-Vp)*Nw=-Vp*(-nx)=Vx
    das2w=-vz(ii)-vr2(ii);      %unit vector of angular acceleration direction in y component
    das3w=vy(ii)-vr3(ii);       %unit vector of angular acceleration direction in z component
    aew=-(Kw.*deltaw.^alpha);   %acceleration of the particle due to elasticity of wall
    afw=Qw*dvnw.*sqrt(deltaw);  %acceleration of the particle due to friction of wall    the directions of aew & afw are related with the normal direction
    aw=aew+afw;                 %total normal acceleration of the particle(the normal direction is the positive direction)
    das23w=sqrt(das2w.^2+das3w.^2);
    mm=das23w==0;        
    das23w(mm)=1;               %to avoid the denominator to be zero
    as2=muw*(das2w./das23w).*abs(aw);
    as3=muw*(das3w./das23w).*abs(aw);
    ax(ii)=ax(ii)-aw;           %ax=ax+aw*Nw=ax-aw
    ay(ii)=ay(ii)-as3;
    az(ii)=az(ii)+as2;
    Fx(ii,Np+1)=-aw;
    Fy(ii,Np+1)=-as3;
    Fz(ii,Np+1)=as2;
    ar2(ii)=ar2(ii)+as2/Io;
    ar3(ii)=ar3(ii)+as3/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
  deltaw=sig/2+(x-W-B+(B-P)/2);
  ii=find((deltaw>=0)  & (yt>=0)); %returns the indices where this is true
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=-vx(ii);             %dvnw=(Vw-Vp)*Nw=-Vp*nx=-Vx
    das2w=vz(ii)-vr2(ii);     %unit vector of angular acceleration direction in y component
    das3w=-vy(ii)-vr3(ii);    %unit vector of angular acceleration direction in z component
    aew=-(Kw.*deltaw.^alpha);   
    afw=Qw*dvnw.*sqrt(deltaw);
    aw=aew+afw;
    das23w=sqrt(das2w.^2+das3w.^2);
    mm=das23w==0;
    das23w(mm)=1;            %to avoid the denominator to be zero
    as2=muw*(das2w./das23w).*abs(aw);
    as3=muw*(das3w./das23w).*abs(aw);
    ax(ii)=ax(ii)+aw;        %ax=ax+aw*Nw=ax+aw
    ay(ii)=ay(ii)+as3;
    az(ii)=az(ii)-as2;
    Fx(ii,Np+2)=aw;
    Fy(ii,Np+2)=as3;
    Fz(ii,Np+2)=-as2;
    ar2(ii)=ar2(ii)+as2/Io;
    ar3(ii)=ar3(ii)+as3/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end

  % z-walls (no contribution to ar3 because there is no spin friction)
  deltaw=Zc-TH/2+sig/2-z;
  ii=find(deltaw>=0); %returns the indices where this is true
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=vz(ii);              %relative velocity of the wall to the particle in normal direction(the normal direction is the positive direction)  dvnw=(Vw-Vp)*Nw=-Vp*(-nz)=-Vz
    das1w=-vy(ii)-vr1(ii);    %unit vector of angular acceleration direction in x component
    das2w=vx(ii)-vr2(ii);     %unit vector of angular acceleration direction in y component
    aew=-(K.*deltaw.^alpha);  %acceleration of the particle due to elasticity of wall
    afw=Qw*dvnw.*sqrt(deltaw); %acceleration of the particle due to friction of wall    the directions of aew & afw are related with the normal direction
    aw=aew+afw;               %total normal acceleration of the particle(the normal direction is the positive direction)
    das12w=sqrt(das1w.^2+das2w.^2);
    mm=das12w==0;        
    das12w(mm)=1;             %to avoid the denominator to be zero
    as1=mu*(das1w./das12w).*abs(aw);
    as2=mu*(das2w./das12w).*abs(aw);
    az(ii)=az(ii)-aw;         %az=az+aw*Nw=az-aw
    ax(ii)=ax(ii)-as2;
    ay(ii)=ay(ii)+as1;
    Fz(ii,Np+3)=-aw;
    Fx(ii,Np+3)=-as2;
    Fy(ii,Np+3)=as1;
    ar1(ii)=ar1(ii)+as1/Io;
    ar2(ii)=ar2(ii)+as2/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
  deltaw=z+sig/2-(Zc+TH/2);
  ii=find(deltaw>=0); 
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=-vz(ii);             %dvnw=(Vw-Vp)*Nw=-Vp*nz=-Vz
    das1w=vy(ii)-vr1(ii);     %unit vector of angular acceleration direction in x component
    das2w=-vx(ii)-vr2(ii);    %unit vector of angular acceleration direction in y component
    aew=-(K.*deltaw.^alpha);   
    afw=Qw*dvnw.*sqrt(deltaw);
    aw=aew+afw;
    das12w=sqrt(das1w.^2+das2w.^2);
    mm=das12w==0;
    das12w(mm)=1;             %to avoid the denominator to be zero
    as1=mu*(das1w./das12w).*abs(aw);
    as2=mu*(das2w./das12w).*abs(aw);
    az(ii)=az(ii)+aw;         %az=az+aw*Nw=az+aw
    ax(ii)=ax(ii)+as2;
    ay(ii)=ay(ii)-as1;
    Fz(ii,Np+4)=aw;
    Fx(ii,Np+4)=as2;
    Fy(ii,Np+4)=-as1;
    ar1(ii)=ar1(ii)+as1/Io;
    ar2(ii)=ar2(ii)+as2/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
    
  % y-walls  (no contribution to ar2)
  deltatp=y+sig/2-ytp;
  ii=find(deltatp>=0); 
  if(~isempty(ii))
    deltatp=2*deltatp(ii);
    dvntp=vytp-vy(ii);             %the relative velocity of the top plate to the particle
    aetp= -(Kw.*deltatp.^alpha);   %acceleration of the particle due to elasticity of the top plate
    aftp=Qw*dvntp.*sqrt(deltatp);  %acceleration of the particle due to friction of the top plate
    atp= aetp+aftp;                %total normal acceleration of the particle(the normal direction is the positive direction)
    aytp = sum(-atp)/MR;  
    das1w=-vz(ii)-vr1(ii);
    das3w=vx(ii)-vr3(ii);
    das13w=sqrt(das1w.^2+das3w.^2);
    mm=das13w==0;
    das13w(mm)=1;                  %to avoid the denominator to be zero
    as1=muw*(das1w./das13w).*abs(atp);
    as3=muw*(das3w./das13w).*abs(atp);
    ay(ii)=ay(ii)+atp;
    ax(ii)=ax(ii)-as3;
    az(ii)=az(ii)+as1;
    Fy(ii,Np+5)=atp;
    Fx(ii,Np+5)=-as3;
    Fz(ii,Np+5)=as1;
    ar1(ii)=ar1(ii)+as1/Io;
    ar3(ii)=ar3(ii)+as3/Io;
    tEWe=tEWe+sum(-aetp.*deltatp/(alpha+1));
  end
  ytps(ntexp)=ytp;
  Ftps(ntexp)=aytp*MR;
%   if(state==2);
%     jj=x(ii)<G*sig0;
%     if(~isempty(jj));
%       ay(ii(jj))=ay(ii(jj))+(cos(3*pi/180)-1)*atp(jj);
%       ax(ii(jj))=ax(ii(jj))-sin(3*pi/180)*atp(jj);
% %       if(ax(ii(jj))./MM(ii(jj))<-2*g)
% %         ax(ii(jj))=-2*g*MM(ii(jj));
% %       end
%     end
%   end
  
  t=t+dt;
  % driving
  ybp=A*(sin(w*t))+A0;
  vybp=A*w*cos(w*t);
  aybp=-A*w*w*sin(w*t);
  deltaw=F+ybp+sig/2-y;
  PP=(nt/npskip>1000)*P;
  ii=find((deltaw>=0) & abs(x-Xc)>(PP)/2); 
  if(~isempty(ii))
    deltaw=2*deltaw(ii);
    dvnw=vy(ii)-vybp;               %dvnw=(vbp-v(ii))*Nw=vy(ii)-vybp
    aew=-(Kw.*deltaw.^alpha);         
    afw=Qwb*dvnw.*sqrt(deltaw);     
    ayw=aew+afw;
    das1w=vz(ii)-vr1(ii);
    das3w=-vx(ii)-vr3(ii);
    das13w=sqrt(das1w.^2+das3w.^2);
    mm=das13w==0;
    das13w(mm)=1;                   %to avoid the denominator to be zero
    as1=muw*(das1w./das13w).*abs(ayw);
    as3=muw*(das3w./das13w).*abs(ayw);
    ay(ii)=ay(ii)-ayw;
    ax(ii)=ax(ii)+as3;
    az(ii)=az(ii)-as1;
    Fy(ii,Np+6)=-ayw;
    Fx(ii,Np+6)=as3;
    Fz(ii,Np+6)=-as1;
    ar1(ii)=ar1(ii)+as1/Io;
    ar3(ii)=ar3(ii)+as3/Io;
    tEWe=tEWe+sum(-aew.*deltaw/(alpha+1));
  end
  
  % find the Cell for each particle.
  NBL=zeros(NBx,NBy,NBz);
  xb=floor(x/scx)+1;
  yb=floor(y/scy);
  zb=floor(z/scz);
  for np=1:Np
    NBL(xb(np),yb(np)+1,zb(np)+1)=NBL(xb(np),yb(np)+1,zb(np)+1)+1;
  end
  cNBL=reshape(cumsum(NBL(:)),NBx,NBy,NBz);
  b=zeros(1,Np);                
  for np=1:Np
    b(cNBL(xb(np)+NBx*yb(np)+NBx*NBy*zb(np)))=np;
    cNBL(xb(np)+NBx*yb(np)+NBx*NBy*zb(np))=cNBL(xb(np)+NBx*yb(np)+NBx*NBy*zb(np))-1;
  end

  
  tEe=0;
  [ixNBL iyNBL izNBL]=find(NBL>0); % indices of cells with at least 1 particle
  for k=1:length(ixNBL)            % loop over all non-empty cells
    n=ixNBL(k);
    m=iyNBL(k);
    l=izNBL(k);
    for q = 1: NBL(n,m,l)      % loop over all particles in cell n,m,l
      for isx = 0:1            % loop over neighbor cells
        idx = n+isx;           % keep neighbors inside
        for isy = -isx:1
          idy = m + isy;
          for isz=min([-isx -isy]):1;
            idz=l+isz;
            if (idx>=1 && idx<=NBx && idy>=1 && idy<=NBy && idz>=1 && idz<=NBz)  %inside range
              cp=b(cNBL(n,m,l)+q); % current particle
              op=b(cNBL(idx,idy,idz)+1+q*(1-isx)*(1-isy)*(1-isz):cNBL(idx,idy,idz)+NBL(idx, idy,idz)); % other particles in box idx,idy,idz
              dx=x(op)-x(cp);  % list of distances between current  particle and all other particles
              dy=y(op)-y(cp);
              dz=z(op)-z(cp);
              dij=sqrt(dx.^2+dy.^2+dz.^2); % distance between particles |Rn-Rm|
              delta=(Dnm(cp,op)-dij);      % distance between particle surfaces
              mm=delta>0;
              if(sum(mm)>0)
                nx=dx./dij;   % Unit vector pointing in the direction of the line between centers
                ny=dy./dij;   % N=(Rn-Rm)/|Rn-Rm|
                nz=dz./dij;

                dvx=vx(op(mm))-vx(cp);  % dV=Vn-Vm
                dvy=vy(op(mm))-vy(cp);
                dvz=vz(op(mm))-vz(cp);

                dvrx=vr1(op(mm))+vr1(cp); % Wn+Wm
                dvry=vr2(op(mm))+vr2(cp);
                dvrz=vr3(op(mm))+vr3(cp);

                dvn=dvx.*nx(mm)+dvy.*ny(mm)+dvz.*nz(mm);         %dVn=dV*N

                das1=ny(mm).*dvz-nz(mm).*dvy+((nx(mm).^2-1).*dvrx+nx(mm).*ny(mm).*dvry+nx(mm).*nz(mm).*dvrz);
                das2=nz(mm).*dvx-nx(mm).*dvz+(nx(mm).*ny(mm).*dvrx+(ny(mm).^2-1).*dvry+ny(mm).*nz(mm).*dvrz);
                das3=nx(mm).*dvy-ny(mm).*dvx+(nx(mm).*nz(mm).*dvrx+ny(mm).*nz(mm).*dvry+(nz(mm).^2-1).*dvrz);
                das123=sqrt(das1.^2+das2.^2+das3.^2);
                das123((das123==0))=1;   %to avoid the denominator to be zero

                ae=-K.*(delta(mm)).^alpha;      % elastic acceleration
                af=Q.*dvn.*sqrt(delta(mm));     % normal friction acceleration
                a=(ae+af);  % like the total acceleration or force magnitude(taking into account the loss due to friction)
               
                % Friction force = mu * Normal force points opposite to
                % surface velocity and is applied to all 3 rotational
                % directions to produce total rotational accelerations
                as1=mu*(das1./das123).*abs(a);
                as2=mu*(das2./das123).*abs(a);
                as3=mu*(das3./das123).*abs(a);

                ar1(cp) = ar1(cp) + sum(as1/Io); %rotational accelerations due to others on current particle
                ar2(cp) = ar2(cp) + sum(as2/Io); %rotational accelerations due to others on current particle
                ar3(cp) = ar3(cp) + sum(as3/Io); %rotational accelerations due to others on current particle
                
                % Newton's 3rd Law
                ar1(op(mm)) = ar1(op(mm)) + as1/Io; %rotational accelerations due to current on other particles
                ar2(op(mm)) = ar2(op(mm)) + as2/Io; %rotational accelerations due to current on other particles
                ar3(op(mm)) = ar3(op(mm)) + as3/Io; %rotational accelerations due to current on other particles

                % total acceleration
                atx=a.*nx(mm)+as2.*nz(mm)-as3.*ny(mm);
                aty=a.*ny(mm)+as3.*nx(mm)-as1.*nz(mm);
                atz=a.*nz(mm)+as1.*ny(mm)-as2.*nx(mm);
                
                % total acceleration
                ax(cp) = ax(cp) + sum(atx);
                ay(cp) = ay(cp) + sum(aty);
                az(cp) = az(cp) + sum(atz);
                
                Fx(op(mm),cp)=Fx(op(mm),cp)-atx';
                Fy(op(mm),cp)=Fy(op(mm),cp)-aty';
                Fz(op(mm),cp)=Fz(op(mm),cp)-atz';

                
                % Newton's 3rd Law
                ax(op(mm)) = ax(op(mm)) - atx; % interaction of current particle with all other particles
                ay(op(mm)) = ay(op(mm)) - aty; % interaction of current particle with all other particles
                az(op(mm)) = az(op(mm)) - atz;
                tEe=tEe+sum(-ae.*delta(mm)/(alpha+1));
              end
            end
          end
        end
      end
    end
  end

  %  aytp=aytp-g; %-beta*g*vytp/A/w;

  up=0;
  if(state==2)
    dytp=(ytp-ytp0);
    if(ntexp-Ntr1-Ntd1<Ntd2/2)
      aytp=aytp-800000*dytp-4000*(vytp+Ly/(Ntd2*dt));
      up=1;
    else
      aytp=aytp-800000*dytp-4000*(vytp-Ly/(Ntd2*dt));
    end
    if(abs(aytp)>10000) aytp=sign(aytp)*10000; end
  else
    aytp=0;
    aytp_old=0;
    vytp=0;
  end
  
  ax=ax./MM;
  ay=ay./MM;
  az=az./MM;

  if(state==2)
    if((dytp>7e-5) && (up==1))
      ax=ax+g*(sin(440*2*pi*t)*(1)+(1+dytp/7e-5)*19*(rand(1,Np)-.5)-1);           % gravitational acceration
      slof=100;
    else
      ax=ax+g*(sin(440*2*pi*t)*(1)-1);           % gravitational acceration
      slof=max(slof-1,1);
    end
  end
  
  
  vx=vx+(ax_old+ax).*dt/2;  % second step in     Verlet integration
  vy=vy+(ay_old+ay).*dt/2;
  vz=vz+(az_old+az).*dt/2;

  Ek(ntexp)=mean(vx.*vx +vy.*vy+vz.*vz)/2;
  mxv(ntexp)=max(abs([vx vy vz]));
  
  if((state==2) && (mxv(ntexp)>5*Ly/Ntd2/dt))
    slof=100;
  else
    slof=max(slof-1,1);
  end
  
  vytp = vytp + (aytp_old + aytp)*dt/2;    

  vr1= vr1 + (ar1_old + ar1).*dt/2;  % rotational velocities
  vr2= vr2 + (ar2_old + ar2).*dt/2;
  vr3= vr3 + (ar3_old + ar3).*dt/2;

  ntf=ntf+1/slof;
  nt=fix(ntf);

  ax_old=ax;
  ay_old=ay;
  az_old=az;
  aytp_old = aytp; 
  aybp_old=aybp;
  ar1_old = ar1;   % update rotational acceleration
  ar2_old = ar2;
  ar3_old = ar3;
end

if(PhotoElastic)
  fclose(fid);
end

%% retry if error

